#include <stdio.h>
#include <stdlib.h>

#include "simple-jpeg/import_export_jpeg.h"

static double average (unsigned char * image_chars,
		       int height, int width) {
  int i, j;
  double sum = 0.0;

  for (i = 0; i < height; i++)
    for (j = 0; j < width; j++)
      sum += image_chars[i*width + j];

  return sum / (height*width);
}

extern void flip_horizontal (unsigned char * image_chars,
			     int height, int width) {
  int i, j;

  for (i = 0; i < height; i++) {
    for (j = 0; j < width/2; j++) {
      unsigned char c = image_chars[i*width + j];
      image_chars[i*width + j] = image_chars[i*width + (width-j-1)];
      image_chars[i*width + (width-j-1)] = c;
    }
  }
}

extern void flip_vertical (unsigned char * image_chars,
			   int height, int width) {
  int i, j;

  for (i = 0; i < height/2; i++) {
    for (j = 0; j < width; j++) {
      unsigned char c = image_chars[i*width + j];
      image_chars[i*width + j] = image_chars[(height-i-1)*width + j];
      image_chars[(height-i-1)*width + j] = c;
    }
  }
}

int main (int argc, char * argv[]) {
  const char * filename;
  unsigned char * image_chars;
  int image_height;
  int image_width;
  int unused;

  if (argc < 2) {
    fprintf(stderr, "usage: %s jpeg-file\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  filename = argv[1];

  import_JPEG_file(filename, &image_chars,
		   &image_height, &image_width,
		   &unused);
  printf("loaded image %s, height: %d, width: %d\n",
	 filename, image_height, image_width);

  printf("average of pixel values: %f\n",
	 average(image_chars, image_height, image_width));

  flip_horizontal(image_chars, image_height, image_width);
  export_JPEG_file("horizontal.jpeg", image_chars,
		   image_height, image_width, unused, 75);
  printf("flipped horizontally to horizontal.jpeg\n");

  /* first flip back to base state. */
  flip_horizontal(image_chars, image_height, image_width);
  flip_vertical(image_chars, image_height, image_width);
  export_JPEG_file("vertical.jpeg", image_chars,
		   image_height, image_width, unused, 75);
  printf("flipped vertically to vertical.jpeg\n");

  free(image_chars);
  exit(EXIT_SUCCESS);
}
